@@ -526,8 +526,11 @@ multi_notebook_notebook_removed (GeditMultiNotebook  *mnb,
 }
 
 static void
-multi_notebook_tabs_reordered (GeditWindow         *window,
-			       GeditDocumentsPanel *panel)
+multi_notebook_tabs_reordered (GeditMultiNotebook  *mnb,
+                               GeditNotebook       *notebook,
+                               GtkWidget           *page,
+                               gint                 page_num,
+                               GeditDocumentsPanel *panel)
 {
 	gedit_debug (DEBUG_PANEL);
 
@@ -562,7 +565,7 @@ set_window (GeditDocumentsPanel *panel,
 			  G_CALLBACK (multi_notebook_tab_removed),
 			  panel);
 	g_signal_connect (panel->priv->mnb,
-			  "tabs-reordered",
+			  "page-reordered",
 			  G_CALLBACK (multi_notebook_tabs_reordered),
 			  panel);
 	g_signal_connect (panel->priv->mnb,
@@ -758,7 +761,7 @@ menu_position (GtkMenu             *menu,
 
 	gdk_window_get_origin (gtk_widget_get_window (w), x, y);
 
-	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
+	gtk_widget_get_preferred_size (GTK_WIDGET (menu), &requisition, NULL);
 
 	gtk_widget_get_allocation (w, &allocation);
 
@@ -1135,4 +1138,4 @@ gedit_documents_panel_new (GeditWindow *window)
 			     NULL);
 }
 
-/* ex:ts=8:noet: */
+/* ex:set ts=8 noet: */
