@@ -79,6 +79,10 @@ struct _GeditMessageType
 	GHashTable *arguments; /* mapping of key -> ArgumentInfo */
 };
 
+G_DEFINE_BOXED_TYPE (GeditMessageType, gedit_message_type,
+                     gedit_message_type_ref,
+                     gedit_message_type_unref)
+
 /**
  * gedit_message_type_ref:
  * @message_type: the #GeditMessageType
@@ -120,28 +124,6 @@ gedit_message_type_unref (GeditMessageType *message_type)
 	g_free (message_type);
 }
 
-/**
- * gedit_message_type_get_type:
- * 
- * Retrieves the GType object which is associated with the
- * #GeditMessageType class.
- * 
- * Return value: the GType associated with #GeditMessageType.
- **/
-GType 
-gedit_message_type_get_type (void)
-{
-	static GType our_type = 0;
-
-	if (!our_type)
-		our_type = g_boxed_type_register_static (
-			"GeditMessageType",
-			(GBoxedCopyFunc) gedit_message_type_ref,
-			(GBoxedFreeFunc) gedit_message_type_unref);
-
-	return our_type;
-}
-
 /**
  * gedit_message_type_identifier:
  * @object_path: (allow-none): the object path
@@ -422,7 +404,7 @@ gedit_message_type_set_valist (GeditMessageType *message_type,
  * Instantiate a new message from the message type with specific values
  * for the message arguments.
  *
- * Return value: the newly created message
+ * Return value: (transfer full): the newly created message
  *
  */
 GeditMessage *
@@ -447,7 +429,7 @@ gedit_message_type_instantiate_valist (GeditMessageType *message_type,
  * Instantiate a new message from the message type with specific values
  * for the message arguments.
  *
- * Return value: the newly created message
+ * Return value: (transfer full): the newly created message
  *
  */
 GeditMessage *
@@ -541,7 +523,7 @@ foreach_gtype (const gchar  *key,
 /**
  * gedit_message_type_foreach:
  * @message_type: the #GeditMessageType
- * @func: the callback function
+ * @func: (scope call): the callback function
  * @user_data: user data supplied to the callback function
  *
  * Calls @func for each argument in the message type.
@@ -559,4 +541,4 @@ gedit_message_type_foreach (GeditMessageType 	    *message_type,
 	g_hash_table_foreach (message_type->arguments, (GHFunc)foreach_gtype, &info);
 }
 
-/* ex:ts=8:noet: */
+/* ex:set ts=8 noet: */
