@@ -62,10 +62,9 @@ struct _GeditNotebookPrivate
 
 	GeditNotebookShowTabsModeType show_tabs_mode;
 
-	gint           xthickness;
+	GtkCssProvider *css;
 
 	guint          close_buttons_sensitive : 1;
-	guint          collapse_border : 1;
 };
 
 G_DEFINE_TYPE(GeditNotebook, gedit_notebook, GTK_TYPE_NOTEBOOK)
@@ -169,6 +168,12 @@ gedit_notebook_dispose (GObject *object)
 		notebook->priv->ui_settings = NULL;
 	}
 
+	if (notebook->priv->css != NULL)
+	{
+		g_object_unref (notebook->priv->css);
+		notebook->priv->css = NULL;
+	}
+
 	G_OBJECT_CLASS (gedit_notebook_parent_class)->dispose (object);
 }
 
@@ -195,20 +200,6 @@ gedit_notebook_grab_focus (GtkWidget *widget)
 	gtk_widget_grab_focus (tab);
 }
 
-static void
-gedit_notebook_style_set (GtkWidget *widget,
-			  GtkStyle  *previous_style)
-{
-	GeditNotebook *nb = GEDIT_NOTEBOOK (widget);
-
-	if (previous_style != NULL && nb->priv->collapse_border)
-	{
-		previous_style->xthickness = 0;
-	}
-
-	GTK_WIDGET_CLASS (gedit_notebook_parent_class)->style_set (widget, previous_style);
-}
-
 /*
  * We need to override this because when we don't show the tabs, like in
  * fullscreen we need to have wrap around too
@@ -409,7 +400,6 @@ gedit_notebook_class_init (GeditNotebookClass *klass)
 	object_class->set_property = gedit_notebook_set_property;
 
 	gtkwidget_class->grab_focus = gedit_notebook_grab_focus;
-	gtkwidget_class->style_set = gedit_notebook_style_set;
 
 	notebook_class->change_current_page = gedit_notebook_change_current_page;
 	notebook_class->switch_page = gedit_notebook_switch_page;
@@ -668,32 +658,54 @@ gedit_notebook_get_close_buttons_sensitive (GeditNotebook *nb)
 
 void
 gedit_notebook_collapse_border (GeditNotebook *nb,
-				gboolean       collapse)
+                                gboolean       collapse)
 {
-	GtkRcStyle *rcstyle;
-	gint xthickness;
+	GtkStyleContext *context;
+	GError *error = NULL;
+	GtkBorder padding;
+	gchar *modified_style;
+	const gchar style[] =
+		".notebook {\n"
+		"	padding: %d 0 %d 0;\n"
+		"}";
 
 	g_return_if_fail (GEDIT_IS_NOTEBOOK (nb));
 
-	nb->priv->collapse_border = collapse;
+	/* FIXME: find out a css like way to do this, right now padding-right/left
+	         doesn't work */
+	context = gtk_widget_get_style_context (GTK_WIDGET (nb));
+	gtk_style_context_get_padding (context, gtk_style_context_get_state (context),
+	                               &padding);
+
+	modified_style = g_strdup_printf (style, padding.top, padding.bottom);
 
 	if (collapse)
 	{
-		GtkStyle *style;
+		/* make it as small as possible */
+		if (nb->priv->css == NULL)
+		{
+			nb->priv->css = gtk_css_provider_new ();
+		}
 
-		style = gtk_widget_get_style (GTK_WIDGET (nb));
-		nb->priv->xthickness = style->xthickness;
-		xthickness = 0;
+		if (gtk_css_provider_load_from_data (nb->priv->css, modified_style, -1, &error))
+		{
+			gtk_style_context_add_provider (context,
+			                                GTK_STYLE_PROVIDER (nb->priv->css),
+			                                GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+		}
+		else
+		{
+			g_warning ("%s", error->message);
+			g_error_free (error);
+		}
 	}
-	else
+	else if (nb->priv->css != NULL)
 	{
-		xthickness = nb->priv->xthickness;
+		gtk_style_context_remove_provider (context,
+		                                   GTK_STYLE_PROVIDER (nb->priv->css));
 	}
 
-	rcstyle = gtk_rc_style_new ();
-	rcstyle->xthickness = xthickness;
-	gtk_widget_modify_style (GTK_WIDGET (nb), rcstyle);
-	g_object_unref (rcstyle);
+	g_free (modified_style);
 }
 
-/* ex:ts=8:noet: */
+/* ex:set ts=8 noet: */
