@@ -51,23 +51,30 @@ struct _GeditOverlayPrivate
 	glong          vadjustment_signal_id;
 
 	GeditTheatricsStage *stage;
+
+	/* GtkScrollablePolicy needs to be checked when
+	 * driving the scrollable adjustment values */
+	guint hscroll_policy : 1;
+	guint vscroll_policy : 1;
 };
 
 enum
 {
 	PROP_0,
-	PROP_MAIN_WIDGET
+	PROP_MAIN_WIDGET,
+	PROP_HADJUSTMENT,
+	PROP_VADJUSTMENT,
+	PROP_HSCROLL_POLICY,
+	PROP_VSCROLL_POLICY
 };
 
-enum
-{
-	SET_SCROLL_ADJUSTMENTS,
-	LAST_SIGNAL
-};
+static void	gedit_overlay_set_hadjustment		(GeditOverlay  *overlay,
+							 GtkAdjustment *adjustment);
+static void	gedit_overlay_set_vadjustment		(GeditOverlay  *overlay,
+							 GtkAdjustment *adjustment);
 
-static guint signals[LAST_SIGNAL] = { 0 };
-
-G_DEFINE_TYPE (GeditOverlay, gedit_overlay, GTK_TYPE_CONTAINER)
+G_DEFINE_TYPE_WITH_CODE (GeditOverlay, gedit_overlay, GTK_TYPE_CONTAINER,
+			 G_IMPLEMENT_INTERFACE (GTK_TYPE_SCROLLABLE, NULL))
 
 static void
 free_container_child (OverlayChild *child)
@@ -142,12 +149,46 @@ gedit_overlay_get_property (GObject    *object,
                             GParamSpec *pspec)
 {
 	GeditOverlay *overlay = GEDIT_OVERLAY (object);
+	GeditOverlayPrivate *priv = overlay->priv;
 
 	switch (prop_id)
 	{
 		case PROP_MAIN_WIDGET:
-			g_value_set_object (value, overlay->priv->main_widget);
+			g_value_set_object (value, priv->main_widget);
 			break;
+
+		case PROP_HADJUSTMENT:
+			g_value_set_object (value, priv->hadjustment);
+			break;
+
+		case PROP_VADJUSTMENT:
+			g_value_set_object (value, priv->vadjustment);
+			break;
+
+		case PROP_HSCROLL_POLICY:
+			if (GTK_IS_SCROLLABLE (priv->main_widget))
+			{
+				g_value_set_enum (value,
+				                  gtk_scrollable_get_hscroll_policy (GTK_SCROLLABLE (priv->main_widget)));
+			}
+			else
+			{
+				g_value_set_enum (value, priv->hscroll_policy);
+			}
+			break;
+
+		case PROP_VSCROLL_POLICY:
+			if (GTK_IS_SCROLLABLE (priv->main_widget))
+			{
+				g_value_set_enum (value,
+				                  gtk_scrollable_get_vscroll_policy (GTK_SCROLLABLE (priv->main_widget)));
+			}
+			else
+			{
+				g_value_set_enum (value, priv->vscroll_policy);
+			}
+			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -161,18 +202,54 @@ gedit_overlay_set_property (GObject      *object,
                             GParamSpec   *pspec)
 {
 	GeditOverlay *overlay = GEDIT_OVERLAY (object);
+	GeditOverlayPrivate *priv = overlay->priv;
 
 	switch (prop_id)
 	{
 		case PROP_MAIN_WIDGET:
-		{
 			overlay->priv->main_widget = g_value_get_object (value);
 			add_toplevel_widget (overlay,
 			                     overlay->priv->main_widget,
 			                     TRUE, FALSE, GDK_GRAVITY_STATIC,
 			                     0);
 			break;
-		}
+
+		case PROP_HADJUSTMENT:
+			gedit_overlay_set_hadjustment (overlay,
+						       g_value_get_object (value));
+			break;
+
+		case PROP_VADJUSTMENT:
+			gedit_overlay_set_vadjustment (overlay,
+						       g_value_get_object (value));
+			break;
+
+		case PROP_HSCROLL_POLICY:
+			if (GTK_IS_SCROLLABLE (priv->main_widget))
+			{
+				gtk_scrollable_set_hscroll_policy (GTK_SCROLLABLE (priv->main_widget),
+				                                   g_value_get_enum (value));
+			}
+			else
+			{
+				priv->hscroll_policy = g_value_get_enum (value);
+				gtk_widget_queue_resize (GTK_WIDGET (overlay));
+			}
+			break;
+
+		case PROP_VSCROLL_POLICY:
+			if (GTK_IS_SCROLLABLE (priv->main_widget))
+			{
+				gtk_scrollable_set_vscroll_policy (GTK_SCROLLABLE (priv->main_widget),
+				                                   g_value_get_enum (value));
+			}
+			else
+			{
+				priv->vscroll_policy = g_value_get_enum (value);
+				gtk_widget_queue_resize (GTK_WIDGET (overlay));
+			}
+			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -186,6 +263,7 @@ gedit_overlay_realize (GtkWidget *widget)
 	GdkWindow *window;
 	GdkWindowAttr attributes;
 	gint attributes_mask;
+	GtkStyleContext *context;
 
 	gtk_widget_set_realized (widget, TRUE);
 
@@ -208,9 +286,63 @@ gedit_overlay_realize (GtkWidget *widget)
 	gtk_widget_set_window (widget, window);
 	gdk_window_set_user_data (window, widget);
 
-	gtk_widget_style_attach (widget);
-	gtk_style_set_background (gtk_widget_get_style (widget), window,
-	                          GTK_STATE_NORMAL);
+	context = gtk_widget_get_style_context (widget);
+	gtk_style_context_set_state (context, GTK_STATE_FLAG_NORMAL);
+	gtk_style_context_set_background (context, window);
+}
+
+static void
+gedit_overlay_get_preferred_width (GtkWidget *widget,
+                                   gint      *minimum,
+                                   gint      *natural)
+{
+	GeditOverlayPrivate *priv = GEDIT_OVERLAY (widget)->priv;
+	OverlayChild *child;
+	GSList *children;
+	gint child_min, child_nat;
+
+	*minimum = 0;
+	*natural = 0;
+
+	for (children = priv->children; children; children = children->next)
+	{
+		child = children->data;
+
+		if (!gtk_widget_get_visible (child->child))
+			continue;
+
+		gtk_widget_get_preferred_width (child->child, &child_min, &child_nat);
+
+		*minimum = MAX (*minimum, child_min);
+		*natural = MAX (*natural, child_nat);
+	}
+}
+
+static void
+gedit_overlay_get_preferred_height (GtkWidget *widget,
+                                    gint      *minimum,
+                                    gint      *natural)
+{
+	GeditOverlayPrivate *priv = GEDIT_OVERLAY (widget)->priv;
+	OverlayChild *child;
+	GSList *children;
+	gint child_min, child_nat;
+
+	*minimum = 0;
+	*natural = 0;
+
+	for (children = priv->children; children; children = children->next)
+	{
+		child = children->data;
+
+		if (!gtk_widget_get_visible (child->child))
+			continue;
+
+		gtk_widget_get_preferred_height (child->child, &child_min, &child_nat);
+
+		*minimum = MAX (*minimum, child_min);
+		*natural = MAX (*natural, child_nat);
+	}
 }
 
 static void
@@ -220,11 +352,12 @@ set_children_positions (GeditOverlay *overlay)
 
 	for (l = overlay->priv->children; l != NULL; l = g_slist_next (l))
 	{
+		GeditOverlayPrivate *priv = overlay->priv;
 		OverlayChild *child = (OverlayChild *)l->data;
 		GtkRequisition req;
 		GtkAllocation alloc;
 
-		if (child->child == overlay->priv->main_widget)
+		if (child->child == priv->main_widget)
 			continue;
 
 		gtk_widget_get_preferred_size (child->child, &req, NULL);
@@ -232,9 +365,13 @@ set_children_positions (GeditOverlay *overlay)
 		/* FIXME: Add all the gravities here */
 		switch (child->gravity)
 		{
-			/* The gravity is the inverse of the place we want */
-			case GDK_GRAVITY_SOUTH_WEST:
-				alloc.x = overlay->priv->main_alloc.width - req.width - child->offset;
+			/* The gravity is treated as position and not as a gravity */
+			case GDK_GRAVITY_NORTH_EAST:
+				alloc.x = priv->main_alloc.width - req.width - child->offset;
+				alloc.y = 0;
+				break;
+			case GDK_GRAVITY_NORTH_WEST:
+				alloc.x = child->offset;
 				alloc.y = 0;
 				break;
 			default:
@@ -244,8 +381,8 @@ set_children_positions (GeditOverlay *overlay)
 
 		if (!child->fixed_position)
 		{
-			alloc.x *= gtk_adjustment_get_value (overlay->priv->hadjustment);
-			alloc.y *= gtk_adjustment_get_value (overlay->priv->vadjustment);
+			alloc.x *= gtk_adjustment_get_value (priv->hadjustment);
+			alloc.y *= gtk_adjustment_get_value (priv->vadjustment);
 		}
 
 		alloc.width = req.width;
@@ -334,46 +471,84 @@ adjustment_value_changed (GtkAdjustment *adjustment,
 }
 
 static void
-gedit_overlay_set_scroll_adjustments (GeditOverlay  *overlay,
-                                      GtkAdjustment *hadjustment,
-                                      GtkAdjustment *vadjustment)
+gedit_overlay_set_hadjustment (GeditOverlay  *overlay,
+                               GtkAdjustment *adjustment)
 {
-	if (overlay->priv->hadjustment != NULL)
+	GeditOverlayPrivate *priv = overlay->priv;
+
+	if (adjustment && priv->vadjustment == adjustment)
+		return;
+
+	if (priv->hadjustment != NULL)
 	{
-		g_signal_handler_disconnect (overlay->priv->hadjustment,
-		                             overlay->priv->hadjustment_signal_id);
+		g_signal_handler_disconnect (priv->hadjustment,
+		                             priv->hadjustment_signal_id);
+		g_object_unref (priv->hadjustment);
 	}
 
-	if (overlay->priv->vadjustment != NULL)
+	if (adjustment == NULL)
 	{
-		g_signal_handler_disconnect (overlay->priv->vadjustment,
-		                             overlay->priv->vadjustment_signal_id);
+		adjustment = gtk_adjustment_new (0.0, 0.0, 0.0,
+		                                 0.0, 0.0, 0.0);
 	}
 
-	if (hadjustment != NULL)
+	priv->hadjustment_signal_id =
+		g_signal_connect (adjustment,
+		                  "value-changed",
+		                  G_CALLBACK (adjustment_value_changed),
+		                  overlay);
+
+	priv->hadjustment = g_object_ref_sink (adjustment);
+
+	if (GTK_IS_SCROLLABLE (priv->main_widget))
 	{
-		overlay->priv->hadjustment_signal_id =
-			g_signal_connect (hadjustment,
-			                  "value-changed",
-			                  G_CALLBACK (adjustment_value_changed),
-			                  overlay);
+		g_object_set (priv->main_widget,
+		              "hadjustment", adjustment,
+		              NULL);
+
 	}
 
-	if (vadjustment != NULL)
+	g_object_notify (G_OBJECT (overlay), "hadjustment");
+}
+
+static void
+gedit_overlay_set_vadjustment (GeditOverlay  *overlay,
+                               GtkAdjustment *adjustment)
+{
+	GeditOverlayPrivate *priv = overlay->priv;
+
+	if (adjustment && priv->vadjustment == adjustment)
+		return;
+
+	if (priv->vadjustment != NULL)
 	{
-		overlay->priv->vadjustment_signal_id =
-			g_signal_connect (vadjustment,
-			                  "value-changed",
-			                  G_CALLBACK (adjustment_value_changed),
-			                  overlay);
+		g_signal_handler_disconnect (priv->vadjustment,
+		                             priv->vadjustment_signal_id);
+		g_object_unref (priv->vadjustment);
 	}
 
-	overlay->priv->hadjustment = hadjustment;
-	overlay->priv->vadjustment = vadjustment;
+	if (adjustment == NULL)
+	{
+		adjustment = gtk_adjustment_new (0.0, 0.0, 0.0,
+		                                 0.0, 0.0, 0.0);
+	}
 
-	gtk_widget_set_scroll_adjustments (overlay->priv->main_widget,
-	                                   hadjustment,
-	                                   vadjustment);
+	overlay->priv->vadjustment_signal_id =
+		g_signal_connect (adjustment,
+		                  "value-changed",
+		                  G_CALLBACK (adjustment_value_changed),
+		                  overlay);
+
+	priv->vadjustment = g_object_ref_sink (adjustment);
+
+	if (GTK_IS_SCROLLABLE (priv->main_widget))
+	{
+		g_object_set (priv->main_widget,
+		              "vadjustment", adjustment,
+		              NULL);
+	}
+
+	g_object_notify (G_OBJECT (overlay), "vadjustment");
 }
 
 static void
@@ -389,6 +564,8 @@ gedit_overlay_class_init (GeditOverlayClass *klass)
 	object_class->set_property = gedit_overlay_set_property;
 
 	widget_class->realize = gedit_overlay_realize;
+	widget_class->get_preferred_width = gedit_overlay_get_preferred_width;
+	widget_class->get_preferred_height = gedit_overlay_get_preferred_height;
 	widget_class->size_allocate = gedit_overlay_size_allocate;
 
 	container_class->add = gedit_overlay_add;
@@ -396,8 +573,6 @@ gedit_overlay_class_init (GeditOverlayClass *klass)
 	container_class->forall = gedit_overlay_forall;
 	container_class->child_type = gedit_overlay_child_type;
 
-	klass->set_scroll_adjustments = gedit_overlay_set_scroll_adjustments;
-
 	g_object_class_install_property (object_class, PROP_MAIN_WIDGET,
 	                                 g_param_spec_object ("main-widget",
 	                                                      "Main Widget",
@@ -407,17 +582,18 @@ gedit_overlay_class_init (GeditOverlayClass *klass)
 	                                                      G_PARAM_CONSTRUCT_ONLY |
 	                                                      G_PARAM_STATIC_STRINGS));
 
-	signals[SET_SCROLL_ADJUSTMENTS] =
-		g_signal_new ("set-scroll-adjustments",
-		              G_OBJECT_CLASS_TYPE (object_class),
-		              G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
-		              G_STRUCT_OFFSET (GeditOverlayClass, set_scroll_adjustments),
-		              NULL, NULL,
-		              gedit_marshal_VOID__OBJECT_OBJECT,
-		              G_TYPE_NONE, 2,
-		              GTK_TYPE_ADJUSTMENT,
-		              GTK_TYPE_ADJUSTMENT);
-	widget_class->set_scroll_adjustments_signal = signals[SET_SCROLL_ADJUSTMENTS];
+	g_object_class_override_property (object_class,
+	                                  PROP_HADJUSTMENT,
+	                                  "hadjustment");
+	g_object_class_override_property (object_class,
+	                                  PROP_VADJUSTMENT,
+	                                  "vadjustment");
+	g_object_class_override_property (object_class,
+	                                  PROP_HSCROLL_POLICY,
+	                                  "hscroll-policy");
+	g_object_class_override_property (object_class,
+	                                  PROP_VSCROLL_POLICY,
+	                                  "vscroll-policy");
 
 	g_type_class_add_private (object_class, sizeof (GeditOverlayPrivate));
 }
@@ -436,7 +612,6 @@ on_actor_step (GeditTheatricsStage *stage,
 	switch (animation_state)
 	{
 		case GEDIT_THEATRICS_ANIMATION_STATE_COMING:
-		{
 			gtk_widget_queue_draw (GTK_WIDGET (anim_widget));
 			gedit_theatrics_animated_widget_set_percent (anim_widget,
 			                                             gedit_theatrics_actor_get_percent (actor));
@@ -446,9 +621,7 @@ on_actor_step (GeditTheatricsStage *stage,
 				                                                     GEDIT_THEATRICS_ANIMATION_STATE_IDLE);
 			}
 			break;
-		}
 		case GEDIT_THEATRICS_ANIMATION_STATE_INTENDING_TO_GO:
-		{
 			gedit_theatrics_animated_widget_set_animation_state (anim_widget,
 			                                                     GEDIT_THEATRICS_ANIMATION_STATE_GOING);
 			gedit_theatrics_animated_widget_set_bias (anim_widget,
@@ -456,77 +629,20 @@ on_actor_step (GeditTheatricsStage *stage,
 			gedit_theatrics_actor_reset (actor, gedit_theatrics_animated_widget_get_duration (anim_widget) *
 			                                    gedit_theatrics_actor_get_percent (actor));
 			break;
-		}
 		case GEDIT_THEATRICS_ANIMATION_STATE_GOING:
-		{
 			if (gedit_theatrics_actor_get_expired (actor))
 			{
-				gtk_container_remove (GTK_CONTAINER (overlay),
-				                      GTK_WIDGET (anim_widget));
+				gtk_widget_destroy (GTK_WIDGET (anim_widget));
 				return;
 			}
+			gtk_widget_queue_draw (GTK_WIDGET (anim_widget));
 			gedit_theatrics_animated_widget_set_percent (anim_widget, 1.0 - gedit_theatrics_actor_get_percent (actor));
 			break;
-		}
 		default:
 			break;
 	}
 }
 
-static void
-remove_core (GeditOverlay                 *overlay,
-             GeditTheatricsAnimatedWidget *aw)
-{
-	GeditTheatricsAnimationState animation_state;
-
-	animation_state = gedit_theatrics_animated_widget_get_animation_state (aw);
-
-	if (animation_state == GEDIT_THEATRICS_ANIMATION_STATE_COMING)
-	{
-		gedit_theatrics_animated_widget_set_animation_state (aw, GEDIT_THEATRICS_ANIMATION_STATE_INTENDING_TO_GO);
-	}
-	else
-	{
-		GeditTheatricsChoreographerEasing easing;
-
-		easing = gedit_theatrics_animated_widget_get_easing (aw);
-
-		switch (easing)
-		{
-			case GEDIT_THEATRICS_CHOREOGRAPHER_EASING_QUADRATIC_IN:
-				gedit_theatrics_animated_widget_set_easing (
-					aw, GEDIT_THEATRICS_CHOREOGRAPHER_EASING_QUADRATIC_OUT);
-				break;
-			case GEDIT_THEATRICS_CHOREOGRAPHER_EASING_QUADRATIC_OUT:
-				gedit_theatrics_animated_widget_set_easing (
-					aw, GEDIT_THEATRICS_CHOREOGRAPHER_EASING_QUADRATIC_IN);
-				break;
-			case GEDIT_THEATRICS_CHOREOGRAPHER_EASING_EXPONENTIAL_IN:
-				gedit_theatrics_animated_widget_set_easing (
-					aw, GEDIT_THEATRICS_CHOREOGRAPHER_EASING_EXPONENTIAL_OUT);
-				break;
-			case GEDIT_THEATRICS_CHOREOGRAPHER_EASING_EXPONENTIAL_OUT:
-				gedit_theatrics_animated_widget_set_easing (
-					aw, GEDIT_THEATRICS_CHOREOGRAPHER_EASING_EXPONENTIAL_IN);
-				break;
-			default:
-				break;
-		}
-
-		gedit_theatrics_animated_widget_set_animation_state (aw, GEDIT_THEATRICS_ANIMATION_STATE_GOING);
-		gedit_theatrics_stage_add_with_duration (overlay->priv->stage,
-		                                         G_OBJECT (aw),
-		                                         gedit_theatrics_animated_widget_get_duration (aw));
-	}
-}
-
-static void
-on_remove_core (GeditTheatricsAnimatedWidget *aw,
-                GeditOverlay                 *overlay)
-{
-	remove_core (overlay, aw);
-}
-
 static void
 gedit_overlay_init (GeditOverlay *overlay)
 {
@@ -548,39 +664,72 @@ gedit_overlay_new (GtkWidget *main_widget)
 	                                 NULL));
 }
 
+static GeditTheatricsAnimatedWidget *
+get_animated_widget (GeditOverlay *overlay,
+                     GtkWidget    *widget)
+{
+	GSList *l;
+
+	for (l = overlay->priv->children; l != NULL; l = g_slist_next (l))
+	{
+		OverlayChild *child = (OverlayChild *)l->data;
+		GtkWidget *in_widget;
+
+		if (!child->is_animated)
+			continue;
+
+		g_object_get (child->child, "widget", &in_widget, NULL);
+		g_assert (in_widget != NULL);
+
+		if (in_widget == widget)
+		{
+			return GEDIT_THEATRICS_ANIMATED_WIDGET (child->child);
+		}
+	}
+
+	return NULL;
+}
+
+/* Note: see that we use the gravity as a position */
 void
-gedit_overlay_add_animated_widget (GeditOverlay                       *overlay,
-                                   GtkWidget                          *widget,
-                                   guint                               duration,
-                                   GeditTheatricsChoreographerEasing   easing,
-                                   GeditTheatricsChoreographerBlocking blocking,
-                                   GtkOrientation                      orientation,
-                                   GdkGravity                          gravity,
-                                   guint                               offset)
+gedit_overlay_slide (GeditOverlay                       *overlay,
+                     GtkWidget                          *widget,
+                     guint                               duration,
+                     GeditTheatricsChoreographerEasing   easing,
+                     GeditTheatricsChoreographerBlocking blocking,
+                     GtkOrientation                      orientation,
+                     GdkGravity                          gravity,
+                     guint                               offset,
+                     gboolean                            in)
 {
 	GeditTheatricsAnimatedWidget *anim_widget;
-	GeditTheatricsActor *actor;
-	GtkAllocation widget_alloc;
 
-	anim_widget = gedit_theatrics_animated_widget_new (widget, duration,
-	                                                   easing,
-	                                                   blocking,
-	                                                   orientation);
-	gtk_widget_show (GTK_WIDGET (anim_widget));
+	anim_widget = get_animated_widget (overlay, widget);
 
-	g_signal_connect (anim_widget,
-	                  "remove-core",
-	                  G_CALLBACK (on_remove_core),
-	                  overlay);
+	if (anim_widget == NULL)
+	{
+		anim_widget = gedit_theatrics_animated_widget_new (widget, duration,
+		                                                   easing,
+		                                                   blocking,
+		                                                   orientation);
+		gtk_widget_show (GTK_WIDGET (anim_widget));
 
-	gtk_widget_get_allocation (widget, &widget_alloc);
-	gedit_theatrics_animated_widget_set_end_padding (anim_widget,
-	                                                 widget_alloc.height);
+		add_toplevel_widget (overlay, GTK_WIDGET (anim_widget), TRUE,
+		                     TRUE, gravity, offset);
+	}
+	else
+	{
+		/* we are only interested in the easing and the blocking */
+		gedit_theatrics_animated_widget_set_easing (anim_widget, easing);
+		gedit_theatrics_animated_widget_set_blocking (anim_widget, blocking);
+	}
 
-	actor = gedit_theatrics_stage_add_with_duration (overlay->priv->stage,
-	                                                 G_OBJECT (anim_widget),
-	                                                 duration);
+	if (!in)
+	{
+		gedit_theatrics_animated_widget_set_animation_state (anim_widget, GEDIT_THEATRICS_ANIMATION_STATE_GOING);
+	}
 
-	add_toplevel_widget (overlay, GTK_WIDGET (anim_widget), TRUE,
-	                     TRUE, gravity, offset);
+	gedit_theatrics_stage_add_with_duration (overlay->priv->stage,
+	                                         G_OBJECT (anim_widget),
+	                                         duration);
 }
