@@ -39,13 +39,6 @@ struct _GeditTheatricsAnimatedWidgetPrivate
 	gdouble bias;
 	gdouble percent;
 	GtkAllocation widget_alloc;
-
-	cairo_surface_t *surface;
-
-	gint width;
-	gint height;
-	gint start_padding;
-	gint end_padding;
 };
 
 enum
@@ -60,14 +53,6 @@ enum
 	PROP_ORIENTATION
 };
 
-enum
-{
-	REMOVE_CORE,
-	LAST_SIGNAL
-};
-
-static guint signals[LAST_SIGNAL] = { 0 };
-
 G_DEFINE_TYPE_EXTENDED (GeditTheatricsAnimatedWidget,
 			gedit_theatrics_animated_widget,
 			GTK_TYPE_BIN,
@@ -75,47 +60,6 @@ G_DEFINE_TYPE_EXTENDED (GeditTheatricsAnimatedWidget,
 			G_IMPLEMENT_INTERFACE (GTK_TYPE_ORIENTABLE,
 					       NULL))
 
-static void
-on_widget_destroyed (GtkWidget                    *widget,
-		     GeditTheatricsAnimatedWidget *aw)
-{
-	GdkWindow *window;
-	cairo_t *img_cr;
-	cairo_t *cr;
-	cairo_surface_t *surface;
-
-	if (!gtk_widget_get_realized (GTK_WIDGET (aw)))
-		return;
-
-	aw->priv->width = aw->priv->widget_alloc.width;
-	aw->priv->height = aw->priv->widget_alloc.height;
-
-	aw->priv->surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
-							aw->priv->width,
-							aw->priv->height);
-
-	img_cr = cairo_create (aw->priv->surface);
-
-	window = gtk_widget_get_window (GTK_WIDGET (aw));
-
-	cr = gdk_cairo_create (GDK_DRAWABLE (window));
-	surface = cairo_get_target (cr);
-
-	cairo_set_source_surface (img_cr, surface,
-	                          aw->priv->widget_alloc.x,
-	                          aw->priv->widget_alloc.y);
-
-	cairo_paint (img_cr);
-
-	cairo_destroy (img_cr);
-	cairo_destroy (cr);
-
-	if (aw->priv->animation_state != GEDIT_THEATRICS_ANIMATION_STATE_GOING)
-	{
-		g_signal_emit (G_OBJECT (aw), signals[REMOVE_CORE], 0);
-	}
-}
-
 static void
 gedit_theatrics_animated_widget_finalize (GObject *object)
 {
@@ -210,12 +154,12 @@ gedit_theatrics_animated_widget_realize (GtkWidget *widget)
 	GdkWindowAttr attributes;
 	GdkWindow *parent_window;
 	GdkWindow *window;
-	GtkStyle *style;
+	GtkStyleContext *context;
 
 	gtk_widget_set_realized (widget, TRUE);
 
 	parent_window = gtk_widget_get_parent_window (widget);
-	style = gtk_widget_get_style (widget);
+	context = gtk_widget_get_style_context (widget);
 
 	attributes.window_type = GDK_WINDOW_CHILD;
 	attributes.wclass = GDK_INPUT_OUTPUT;
@@ -224,50 +168,70 @@ gedit_theatrics_animated_widget_realize (GtkWidget *widget)
 	window = gdk_window_new (parent_window, &attributes, 0);
 	gdk_window_set_user_data (window, widget);
 	gtk_widget_set_window (widget, window);
-	style = gtk_style_attach (style, window);
-	gtk_widget_set_style (widget, style);
-	gtk_style_set_background (style, window, GTK_STATE_NORMAL);
+	gtk_style_context_set_state (context, GTK_STATE_FLAG_NORMAL);
+	gtk_style_context_set_background (context, window);
 }
 
 static void
-gedit_theatrics_animated_widget_size_request (GtkWidget      *widget,
-					      GtkRequisition *requisition)
+gedit_theatrics_animated_widget_get_preferred_width (GtkWidget *widget,
+                                                     gint      *minimum,
+                                                     gint      *natural)
 {
 	GeditTheatricsAnimatedWidget *aw = GEDIT_THEATRICS_ANIMATED_WIDGET (widget);
 	gint width;
-	gint height;
 
 	if (aw->priv->widget != NULL)
 	{
-		GtkRequisition req;
+		gint child_min, child_nat;
 
-		gtk_widget_get_preferred_size (aw->priv->widget,
-		                               &req, NULL);
-		aw->priv->widget_alloc.width = req.width;
-		aw->priv->widget_alloc.height = req.height;
+		gtk_widget_get_preferred_width (aw->priv->widget,
+		                                &child_min, &child_nat);
+		aw->priv->widget_alloc.width = child_min;
 	}
 
 	if (aw->priv->orientation == GTK_ORIENTATION_HORIZONTAL)
 	{
 		width = gedit_theatrics_choreographer_pixel_compose (aw->priv->percent,
-								     aw->priv->widget_alloc.width +
-								     aw->priv->start_padding +
-								     aw->priv->end_padding,
-								     aw->priv->easing);
-		height = aw->priv->widget_alloc.height;
+		                                                     aw->priv->widget_alloc.width,
+		                                                     aw->priv->easing);
 	}
 	else
 	{
 		width = aw->priv->widget_alloc.width;
-		height = gedit_theatrics_choreographer_pixel_compose (aw->priv->percent,
-								      aw->priv->widget_alloc.height +
-								      aw->priv->start_padding +
-								      aw->priv->end_padding,
-								      aw->priv->easing);
 	}
 
-	requisition->width = width;
-	requisition->height = height;
+	*minimum = *natural = width;
+}
+
+static void
+gedit_theatrics_animated_widget_get_preferred_height (GtkWidget *widget,
+                                                      gint      *minimum,
+                                                      gint      *natural)
+{
+	GeditTheatricsAnimatedWidget *aw = GEDIT_THEATRICS_ANIMATED_WIDGET (widget);
+	gint height;
+
+	if (aw->priv->widget != NULL)
+	{
+		gint child_min, child_nat;
+
+		gtk_widget_get_preferred_height (aw->priv->widget,
+		                                 &child_min, &child_nat);
+		aw->priv->widget_alloc.height = child_min;
+	}
+
+	if (aw->priv->orientation == GTK_ORIENTATION_HORIZONTAL)
+	{
+		height = aw->priv->widget_alloc.height;
+	}
+	else
+	{
+		height = gedit_theatrics_choreographer_pixel_compose (aw->priv->percent,
+		                                                      aw->priv->widget_alloc.height,
+		                                                      aw->priv->easing);
+	}
+
+	*minimum = *natural = height;
 }
 
 static void
@@ -283,17 +247,17 @@ gedit_theatrics_animated_widget_size_allocate (GtkWidget     *widget,
 		if (aw->priv->orientation == GTK_ORIENTATION_HORIZONTAL)
 		{
 			aw->priv->widget_alloc.height = allocation->height;
-			aw->priv->widget_alloc.x = aw->priv->start_padding;
+			aw->priv->widget_alloc.x = 0;
 
 			if (aw->priv->blocking == GEDIT_THEATRICS_CHOREOGRAPHER_BLOCKING_DOWNSTAGE)
 			{
-				aw->priv->widget_alloc.x += allocation->width - aw->priv->widget_alloc.width;
+				aw->priv->widget_alloc.x = allocation->width - aw->priv->widget_alloc.width;
 			}
 		}
 		else
 		{
 			aw->priv->widget_alloc.width = allocation->width;
-			aw->priv->widget_alloc.y = aw->priv->start_padding;
+			aw->priv->widget_alloc.y = 0;
 
 			if (aw->priv->blocking == GEDIT_THEATRICS_CHOREOGRAPHER_BLOCKING_DOWNSTAGE)
 			{
@@ -309,33 +273,6 @@ gedit_theatrics_animated_widget_size_allocate (GtkWidget     *widget,
 	}
 }
 
-static gboolean
-gedit_theatrics_animated_widget_draw (GtkWidget      *widget,
-				      cairo_t        *cr)
-{
-	GeditTheatricsAnimatedWidget *aw = GEDIT_THEATRICS_ANIMATED_WIDGET (widget);
-
-	if (aw->priv->surface != NULL)
-	{
-		/* Do not scale if the size is 0 */
-		if (aw->priv->width > 0 && aw->priv->height > 0)
-		{
-			cairo_scale (cr,
-				     aw->priv->widget_alloc.width / aw->priv->width,
-				     aw->priv->widget_alloc.height / aw->priv->height);
-			cairo_set_source_surface (cr, aw->priv->surface, 0, 0);
-
-			cairo_paint (cr);
-		}
-
-		return TRUE;
-	}
-	else
-	{
-		return GTK_WIDGET_CLASS (gedit_theatrics_animated_widget_parent_class)->draw (widget, cr);
-	}
-}
-
 static void
 gedit_theatrics_animated_widget_add (GtkContainer *container,
 				     GtkWidget    *widget)
@@ -344,10 +281,6 @@ gedit_theatrics_animated_widget_add (GtkContainer *container,
 
 	aw->priv->widget = widget;
 
-	g_signal_connect (widget, "destroy",
-			  G_CALLBACK (on_widget_destroyed),
-			  aw);
-
 	GTK_CONTAINER_CLASS (gedit_theatrics_animated_widget_parent_class)->add (container, widget);
 }
 
@@ -374,9 +307,9 @@ gedit_theatrics_animated_widget_class_init (GeditTheatricsAnimatedWidgetClass *k
 	object_class->set_property = gedit_theatrics_animated_widget_set_property;
 
 	widget_class->realize = gedit_theatrics_animated_widget_realize;
-	widget_class->size_request = gedit_theatrics_animated_widget_size_request;
+	widget_class->get_preferred_width = gedit_theatrics_animated_widget_get_preferred_width;
+	widget_class->get_preferred_height = gedit_theatrics_animated_widget_get_preferred_height;
 	widget_class->size_allocate = gedit_theatrics_animated_widget_size_allocate;
-	widget_class->draw = gedit_theatrics_animated_widget_draw;
 
 	container_class->add = gedit_theatrics_animated_widget_add;
 	container_class->remove = gedit_theatrics_animated_widget_remove;
@@ -446,16 +379,6 @@ gedit_theatrics_animated_widget_class_init (GeditTheatricsAnimatedWidgetClass *k
 	                                  PROP_ORIENTATION,
 	                                  "orientation");
 
-	signals[REMOVE_CORE] =
-		g_signal_new ("remove-core",
-		              G_OBJECT_CLASS_TYPE (object_class),
-		              G_SIGNAL_RUN_LAST,
-		              G_STRUCT_OFFSET (GeditTheatricsAnimatedWidgetClass, remove_core),
-		              NULL, NULL,
-		              g_cclosure_marshal_VOID__VOID,
-		              G_TYPE_NONE,
-		              0);
-
 	g_type_class_add_private (object_class, sizeof (GeditTheatricsAnimatedWidgetPrivate));
 }
 
@@ -607,13 +530,4 @@ gedit_theatrics_animated_widget_set_bias (GeditTheatricsAnimatedWidget *aw,
 	aw->priv->bias = bias;
 }
 
-void
-gedit_theatrics_animated_widget_set_end_padding (GeditTheatricsAnimatedWidget *aw,
-                                                 gint                          end_padding)
-{
-	g_return_if_fail (GEDIT_IS_THEATRICS_ANIMATED_WIDGET (aw));
-
-	aw->priv->end_padding = end_padding;
-}
-
-/* ex:ts=8:noet: */
+/* ex:set ts=8 noet: */
