@@ -57,7 +57,7 @@ class PythonConsole(Gtk.ScrolledWindow):
         self._interface_settings = Gio.Settings.new(self.SETTINGS_INTERFACE_DIR)
         self._interface_settings.connect("changed", self.on_settings_changed)
 
-        self._profile_settings = Gio.Settings.new(self.SETTINGS_PROFILE_DIR)
+        self._profile_settings = self.get_profile_settings()
         self._profile_settings.connect("changed", self.on_settings_changed)
 
         self.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
@@ -101,6 +101,10 @@ class PythonConsole(Gtk.ScrolledWindow):
         self.view.connect("key-press-event", self.__key_press_event_cb)
         buf.connect("mark-set", self.__mark_set_cb)
 
+    def get_profile_settings(self):
+        #FIXME return either the gnome-terminal settings or the gedit one
+        return Gio.Settings.new(self.CONSOLE_KEY_BASE)
+
     def do_grab_focus(self):
         self.view.grab_focus()
 
@@ -120,13 +124,13 @@ class PythonConsole(Gtk.ScrolledWindow):
             if font_name != self.DEFAULT_FONT:
                 if font_name != system_font:
                     try:
-                        font_desc = pango.FontDescription(system_font)
+                        font_desc = Pango.FontDescription(system_font)
                     except:
                         pass
 
                 if font_desc == None:
                     try:
-                        font_desc = pango.FontDescription(self.DEFAULT_FONT)
+                        font_desc = Pango.FontDescription(self.DEFAULT_FONT)
                     except:
                         pass
 
@@ -147,10 +151,10 @@ class PythonConsole(Gtk.ScrolledWindow):
         modifier_mask = Gtk.accelerator_get_default_mod_mask()
         event_state = event.key.state & modifier_mask
 
-        if event.key.keyval == Gdk.d and event_state == Gdk.ModifierType.CONTROL_MASK:
+        if event.key.keyval == Gdk.KEY_D and event_state == Gdk.ModifierType.CONTROL_MASK:
             self.destroy()
 
-        elif event.key.keyval == Gdk.Return and event_state == Gdk.ModifierType.CONTROL_MASK:
+        elif event.key.keyval == Gdk.KEY_Return and event_state == Gdk.ModifierType.CONTROL_MASK:
             # Get the command
             buf = view.get_buffer()
             inp_mark = buf.get_mark("input")
@@ -176,7 +180,7 @@ class PythonConsole(Gtk.ScrolledWindow):
             GObject.idle_add(self.scroll_to_end)
             return True
 
-        elif event.key.keyval == Gdk.Return:
+        elif event.key.keyval == Gdk.KEY_Return:
             # Get the marks
             buf = view.get_buffer()
             lin_mark = buf.get_mark("input-line")
@@ -220,22 +224,22 @@ class PythonConsole(Gtk.ScrolledWindow):
             GObject.idle_add(self.scroll_to_end)
             return True
 
-        elif event.key.keyval == Gdk.KP_Down or event.key.keyval == Gdk.Down:
+        elif event.key.keyval == Gdk.KEY_KP_Down or event.key.keyval == Gdk.KEY_Down:
             # Next entry from history
             view.emit_stop_by_name("key_press_event")
             self.history_down()
             GObject.idle_add(self.scroll_to_end)
             return True
 
-        elif event.key.keyval == Gdk.KP_Up or event.key.keyval == Gdk.Up:
+        elif event.key.keyval == Gdk.KEY_KP_Up or event.key.keyval == Gdk.KEY_Up:
             # Previous entry from history
             view.emit_stop_by_name("key_press_event")
             self.history_up()
             GObject.idle_add(self.scroll_to_end)
             return True
 
-        elif event.key.keyval == Gdk.KP_Left or event.key.keyval == Gdk.Left or \
-             event.key.keyval == Gdk.BackSpace:
+        elif event.key.keyval == Gdk.KEY_KP_Left or event.key.keyval == Gdk.KEY_Left or \
+             event.key.keyval == Gdk.KEY_BackSpace:
             buf = view.get_buffer()
             inp = buf.get_iter_at_mark(buf.get_mark("input"))
             cur = buf.get_iter_at_mark(buf.get_insert())
@@ -248,7 +252,7 @@ class PythonConsole(Gtk.ScrolledWindow):
         # For the console we enable smart/home end behavior incoditionally
         # since it is useful when editing python
 
-        elif (event.key.keyval == Gdk.KP_Home or event.key.keyval == Gdk.Home) and \
+        elif (event.key.keyval == Gdk.KEY_KP_Home or event.key.keyval == Gdk.KEY_Home) and \
              event_state == event_state & (Gdk.ModifierType.SHIFT_MASK|Gdk.ModifierType.CONTROL_MASK):
             # Go to the begin of the command instead of the begin of the line
             buf = view.get_buffer()
@@ -267,7 +271,7 @@ class PythonConsole(Gtk.ScrolledWindow):
                 buf.place_cursor(it)
             return True
 
-        elif (event.key.keyval == Gdk.KP_End or event.key.keyval == Gdk.End) and \
+        elif (event.key.keyval == Gdk.KEY_KP_End or event.key.keyval == Gdk.KEY_End) and \
              event_state == event_state & (Gdk.ModifierType.SHIFT_MASK|Gdk.ModifierType.CONTROL_MASK):
 
             buf = view.get_buffer()
@@ -335,15 +339,11 @@ class PythonConsole(Gtk.ScrolledWindow):
 
     def write(self, text, tag = None):
         buf = self.view.get_buffer()
-        #FIXME
-        #if tag is None:
-        #    buf.insert(buf.get_end_iter(), text)
-        #else:
-        #    buf.insert_with_tags(buf.get_end_iter(), text, tag)
-        insertion_iter = buf.get_end_iter()
-        buf.insert(insertion_iter, text)
-        if tag is not None:
-            buf.apply_tag(tag, insertion_iter, buf.get_end_iter())
+        if tag is None:
+            buf.insert(buf.get_end_iter(), text)
+        else:
+            buf.insert_with_tags(buf.get_end_iter(), text, tag)
+
         GObject.idle_add(self.scroll_to_end)
 
     def eval(self, command, display_command = False):
