@@ -20,6 +20,7 @@ import os
 import shutil
 
 import cairo
+import glib
 
 from gi.repository import Gtk, Gdk, Gedit, PeasGtk, GObject
 import platform
@@ -33,7 +34,7 @@ from windowactivatable import WindowActivatable
 class AppActivatable(GObject.Object, Gedit.AppActivatable, PeasGtk.Configurable):
         __gtype_name__ = "GeditSnippetsAppActivatable"
 
-        app = GObject.property(type=GObject.Object)
+        app = GObject.property(type=Gedit.App)
 
         def __init__(self):
                 GObject.Object.__init__(self)
@@ -52,7 +53,7 @@ class AppActivatable(GObject.Object, Gedit.AppActivatable, PeasGtk.Configurable)
                         if userdir:
                                 snippetsdir = os.path.join(userdir, 'gedit/snippets')
                         else:
-                                snippetsdir = os.path.expanduser('~/.gnome2/gedit/snippets')
+                                snippetsdir = os.path.join(glib.get_user_config_dir(), 'gedit/snippets')
 
                 library.set_dirs(snippetsdir, self.system_dirs())
 
